@@ -867,6 +867,26 @@ class XCCClient(IMMClient):
             raise pygexc.UnsupportedFunctionality(
                 'This platform does not support AC reseat.')
 
+    def fetch_dimm(self, name, fru):
+        meminfo = self.grab_cacheable_json('/api/dataset/imm_memory')
+        meminfo = meminfo.get('items', [{}])[0].get('memory', [])
+        for memi in meminfo:
+            if memi.get('memory_description', None) == name:
+                fru['model'] = memi['memory_part_number']
+                fru['ecc'] = memi.get('memory_ecc_bits', 0) != 0
+                fru['manufacture_location'] = 0
+                fru['memory_type'] = memi['memory_type']
+                fru['module_type'] = fru['memory_type']
+                mdate = memi['memory_manuf_date']
+                mdate = '20{}-W{}'.format(mdate[-2:], mdate[:-2])
+                fru['manufacture_date'] = mdate
+                speed = memi['memory_config_speed'] * 8 / 100 * 100
+                fru['speed'] = speed
+                fru['capacity_mb'] = memi['memory_capacity'] * 1024
+                fru['serial'] = memi['memory_serial_number'].strip()
+                fru['manufacturer'] = memi['memory_manufacturer']
+                break
+
     def get_description(self):
         dsc = self.wc.grab_json_response('/DeviceDescription.json')
         dsc = dsc[0]
@@ -1019,6 +1039,11 @@ class XCCClient(IMMClient):
                    'Referer': 'https://xcc/',
                    'Host': 'xcc',
                    'Content-Type': 'application/json'}
+        rsp, status = wc.grab_json_response_with_status(
+            '/api/providers/get_nonce', {})
+        if status == 200:
+            nonce = rsp.get('nonce', None)
+            headers['Content-Security-Policy'] = 'nonce={0}'.format(nonce)
         wc.request('POST', '/api/login', adata, headers)
         rsp = wc.getresponse()
         if rsp.status == 200:
@@ -1344,6 +1369,10 @@ class XCCClient(IMMClient):
                 write_policy = vol.write_policy
             else:
                 write_policy = props["cpwb"]
+            if vol.default_init is not None:
+                default_init = vol.default_init
+            else:
+                default_init = props["initstate"]
             strsize = 'remainder' if vol.size is None else str(vol.size)
             if strsize in ('all', '100%'):
                 volsize = params['capacity']
@@ -1364,7 +1393,7 @@ class XCCClient(IMMClient):
                     'Requested sizes exceed available capacity')
             vols.append('{0};{1};{2};{3};{4};{5};{6};{7};{8};|'.format(
                 name, volsize, stripsize, write_policy, read_policy,
-                props['cpio'], props['ap'], props['dcp'], props['initstate']))
+                props['cpio'], props['ap'], props['dcp'], default_init))
         url = '/api/function'
         cid = params['controller'].split(',')
         cnum = cid[0]
@@ -1473,9 +1502,11 @@ class XCCClient(IMMClient):
                             spares.append(diskinfo)
                         else:
                             disks.append(diskinfo)
-                    totalsize = pool['totalCapacityStr'].replace('GB', '')
+                    totalsize = pool['totalCapacityStr'].replace(
+                        'GB', '').replace('GiB', '')
                     totalsize = int(float(totalsize) * 1024)
-                    freesize = pool['freeCapacityStr'].replace('GB', '')
+                    freesize = pool['freeCapacityStr'].replace(
+                        'GB', '').replace('GiB', '')
                     freesize = int(float(freesize) * 1024)
                     pools.append(storage.Array(
                         disks=disks, raid=pool['rdlvlstr'], volumes=volumes,
@@ -1538,6 +1569,8 @@ class XCCClient(IMMClient):
             self.fwo = None
         if self._keepalivesession:
             self._refresh_token_wc(self._keepalivesession)
+        elif self._wc and self._wc.vintage < util._monotonic_time() - 20:
+            self.weblogout()
 
     def fetch_psu_firmware(self):
         psudata = self.get_cached_data('lenovo_cached_psu')
