@@ -228,7 +228,7 @@ fpc_sensors = {
         'type': 'Fan',
         'units': 'RPM',
         'provider': fpc_read_psu_fan,
-        'elements': 1,
+        'elementsfun': get_psu_count,
     },
     'Total Power Capacity': {
         'type': 'Power',
@@ -326,10 +326,12 @@ def get_fpc_firmware(bmcver, ipmicmd, fpcorsmm):
     if fpcorsmm != 6:  # SMM
         if fpcorsmm >> 5:
             name = 'SMM2'
+            buildid = '{0}{1}{2}{3}{4}{5}{6}'.format(
+                *[chr(x) for x in builddata[6:13]])
         else:
             name = 'SMM'
-        buildid = '{0}{1}{2}{3}{4}{5}{6}'.format(
-            *[chr(x) for x in builddata[6:13]])
+            buildid = '{0}{1}{2}{3}{4}{5}{6}'.format(
+                *[chr(x) for x in builddata[5:12]])
     elif len(builddata) == 8:
         builddata = builddata[1:]  # discard the 'completion code'
         name = 'FPC'
@@ -882,21 +884,27 @@ class SMMClient(object):
         psui['Revision'] = psuinfo[34]
         psui['Description'] = psutypes.get(
             psutype, 'Unknown ({})'.format(psutype))
-        psui['Part Number'] = str(psuinfo[35:47].strip().decode('utf8'))
-        psui['FRU Number'] = str(psuinfo[47:59].strip().decode('utf8'))
-        psui['Serial Number'] = str(psuinfo[59:71].strip().decode('utf8'))
-        psui['Header Code'] = str(psuinfo[71:75].strip().decode('utf8'))
-        psui['Vendor'] = str(psuinfo[25:29].strip().decode('utf8'))
+        psui['Part Number'] = str(psuinfo[35:47].strip(
+            b' \x00\xff').decode('utf8'))
+        psui['FRU Number'] = str(psuinfo[47:59].strip(
+            b' \x00\xff').decode('utf8'))
+        psui['Serial Number'] = str(psuinfo[59:71].strip(
+            b' \x00\xff').decode('utf8'))
+        psui['Header Code'] = str(psuinfo[71:75].strip(
+            b' \x00\xff').decode('utf8'))
+        psui['Vendor'] = str(psuinfo[25:29].strip(
+            b' \x00\xff').decode('utf8'))
         psui['Manufacturing Date'] = '20{}-W{}'.format(
             psuinfo[77:79].decode('utf8'), psuinfo[75:77].decode('utf8'))
         psui['Primary Firmware Version'] = '{:x}.{:x}'.format(
             psuinfo[80], psuinfo[79])
         psui['Secondary Firmware Version'] = '{:x}.{:x}'.format(
             psuinfo[82], psuinfo[81])
-        psui['Model'] = str(psuinfo[5:23].strip().decode('utf8'))
+        psui['Model'] = str(psuinfo[5:23].strip(b' \x00\xff').decode('utf8'))
         psui['Manufacturer Location'] = str(
-            psuinfo[83:85].strip().decode('utf8'))
-        psui['Barcode'] = str(psuinfo[85:108].strip().decode('utf8'))
+            psuinfo[83:85].strip(b' \x00\xff').decode('utf8'))
+        psui['Barcode'] = str(psuinfo[85:108].strip(
+            b' \x00\xff').decode('utf8'))
         return psui
 
     def logout(self):
