@@ -308,9 +308,11 @@ class OEMHandler(generic.OEMHandler):
                             spares.append(diskinfo)
                         else:
                             disks.append(diskinfo)
-                    totalsize = pool['totalCapacityStr'].replace('GB', '')
+                    totalsize = pool['totalCapacityStr'].replace(
+                        'GB', '').replace('GiB', '')
                     totalsize = int(float(totalsize) * 1024)
-                    freesize = pool['freeCapacityStr'].replace('GB', '')
+                    freesize = pool['freeCapacityStr'].replace(
+                        'GB', '').replace('GiB', '')
                     freesize = int(float(freesize) * 1024)
                     pools.append(storage.Array(
                         disks=disks, raid=pool['rdlvlstr'], volumes=volumes,
@@ -513,6 +515,10 @@ class OEMHandler(generic.OEMHandler):
                 write_policy = vol.write_policy
             else:
                 write_policy = props["cpwb"]
+            if vol.default_init is not None:
+                default_init = vol.default_init
+            else:
+                default_init = props["initstate"]
             strsize = 'remainder' if vol.size is None else str(vol.size)
             if strsize in ('all', '100%'):
                 volsize = params['capacity']
@@ -534,7 +540,7 @@ class OEMHandler(generic.OEMHandler):
                     'Requested sizes exceed available capacity')
             vols.append('{0};{1};{2};{3};{4};{5};{6};{7};{8};|'.format(
                 name, volsize, stripsize, write_policy, read_policy,
-                props['cpio'], props['ap'], props['dcp'], props['initstate']))
+                props['cpio'], props['ap'], props['dcp'], default_init))
         url = '/api/function'
         cid = params['controller'].split(',')
         cnum = cid[0]
@@ -622,6 +628,11 @@ class OEMHandler(generic.OEMHandler):
                    'Referer': referer,
                    'Host': 'xcc',
                    'Content-Type': 'application/json'}
+        rsp, status = wc.grab_json_response_with_status(
+            '/api/providers/get_nonce', {})
+        if status == 200:
+            nonce = rsp.get('nonce', None)
+            headers['Content-Security-Policy'] = 'nonce={0}'.format(nonce)
         wc.request('POST', '/api/login', adata, headers)
         rsp = wc.getresponse()
         if rsp.status == 200:
